-------------------------------------------------------------------
-- Date			: 21-MAY-2012
-- Database		: SQLServer 2005/2008
-- Description	: ISAMESSO user domain migration script
-- IMS Version	: 8.1.x
--
-- Dependencies :
--	 - ISAMESSO Enterprise ID / Domain Update Script (sp_base_updEntIdDomainUpnGuid.sql)
--
-- Assumption :-
--   - NEW domain is configured in ISAMESSO
--   - User account in enterprise directory is migrated in new domain  
--
-- Notes :-
--   - Changes on user domain will requires manual changes on the new access profile that rely on enterprise directory
--   - Changes on enterprise id / user domain will trigger password change flow at AA's side
--	 - Change of UPN will affects :
--		 -- Smart Card (00D)
--		 -- AA/AAWWP login with UPN
--
-- Tables affected : 
--  - IMSIdentityAttribute 
--  - IMSIdentityUniqueAttribute
--	- IMSTemplatedSyncRepo
--	- IMSSoci 
--	- IMSSociAttribute
--	- IMSSociUniqueAttribute
--  - IMSCertificate 
--
-- drop procedure sp_migrateUserDomain
-- exec sp_migrateUserDomain 'SIMULATE', 'hlong-test02', 'qa.encentuate.com', 'song.ims82.com', '{B4570C0D-33D0-4B0F-B6B6-CAD9226F9EE9}', 'hlong-test02@song.ims82.com'
-- exec sp_migrateUserDomain 'SIMULATE', 'hlong-test02', 'song.ims82.com', 'qa.encentuate.com', '{987F61DB-C682-4836-A5D4-7E6B447CC719}', 'hlong-test02@qa.encentuate.com'
-- exec sp_migrateUserDomain 'SIMULATE', 'hlong-test02', 'qa.encentuate.com', 'qa.encentuate.com', NULL, NULL
-- exec sp_migrateUserDomain 'ABOUT', '', '', '', NULL, NULL
-- exec sp_migrateUserDomain 'HELP', '', '', '',  NULL, NULL
--
--
CREATE PROCEDURE sp_migrateUserDomain
 @IN_operationCmd VARCHAR(8)
 
,@IN_curUserEntId VARCHAR(128)
,@IN_curDomain VARCHAR(128)

,@IN_newDomain VARCHAR(128)
,@IN_newADGUID VARCHAR(128)
,@IN_newUPN VARCHAR(128)
AS
BEGIN
	DECLARE @var_showSyntax BIT; SET @var_showSyntax = 0;
	DECLARE @var_scriptName VARCHAR(64); SET @var_scriptName = 'sp_migrateUserDomain';
	DECLARE @var_scriptDesc VARCHAR(64); SET @var_scriptDesc = 'User Domain Migration';
	DECLARE @var_scriptVer VARCHAR(32); SET @var_scriptVer = '1.0.0';
	
	SET NOCOUNT ON;
	
	DECLARE @var_new_upn VARCHAR(256);
	DECLARE @var_new_ad_userguid VARCHAR(256);

	IF(RTRIM(@IN_newUPN) = '')
		SET @var_new_upn = NULL;
	ELSE 
		SET @var_new_upn = @IN_newUPN;

	IF(RTRIM(@IN_newADGUID) = '')
		SET @var_new_ad_userguid = NULL;
	ELSE 
		SET @var_new_ad_userguid = @IN_newADGUID;

	PRINT N'command : sp_migrateUserDomain '''+@IN_operationCmd+''', '''+@IN_curUserEntId+''', '''+@IN_curDomain+''', '''+@IN_newDomain+''', '+COALESCE(''''+@IN_newADGUID+'''','NULL')+', '+COALESCE(''''+@IN_newUPN+'''','NULL');

	IF(OBJECT_ID('sp_base_updEntIdDomainUpnGuid')IS NULL)
	BEGIN
		PRINT N'   [ERROR] Missing dependency. Please make sure ISAMESSO stored procedure ''sp_base_updEntIdDomainUpnGuid'' is created in the same schema.';
	END ELSE
	BEGIN
	
		exec sp_base_updEntIdDomainUpnGuid @IN_operationCmd, @IN_curUserEntId, @IN_curDomain, @IN_curUserEntId, @IN_newDomain , @var_new_upn, @var_new_ad_userguid, @var_showSyntax OUTPUT, @var_scriptName, @var_scriptDesc, @var_scriptVer
		
		IF @var_showSyntax = 1
		BEGIN
			PRINT N' syntax : sp_migrateUserDomain ''<mode>'', ''<user id>'', ''<current user domain>'', ''<new user domain>'', ''<new AD userGUID>'', ''<new upn>''';
			
			PRINT N' * <mode> - Mandatory, text, case sensitive'; 
			PRINT N' 		- ''ABOUT'' display script information';
			PRINT N' 		- ''HELP'' display script syntax and example';
			PRINT N' 		- ''UPDATE'' to perform actual rename';
			PRINT N' 		- ''SIMULATE'' to check on the target enterprise user id without performing actual migration'; 
			PRINT N' * <user id> 	- Mandatory, text, case sensitive';
			PRINT N'          		- Target user enterprise ID';
			PRINT N' * <current user domain> - Mandatory, text, case sensitive';
			PRINT N' 	           			- current user domain';
			PRINT N' * <new user domain> 	- Mandatory, text, case sensitive';
			PRINT N' 	           			- new user domain';
			PRINT N' * <new AD userGUID> 	- Optional, text, case sensitive';
			PRINT N' 	           			- Applicable for Active Directory only';
			PRINT N' 	           			- Enter ''NULL'' if not applicable or value remain unchanged';
			PRINT N' 	           			- New active directory user account objectGUID value';
			PRINT N' * <new upn> 	- Mandatory, text, case sensitive';
			PRINT N'   				- Enter ''NULL'' if not applicable or value remain unchanged';
			PRINT N'';
			PRINT N'Example : ';
			PRINT N' * sp_migrateUserDomain ''SIMULATE'',''ims001'', ''current.domain.com'', ''new.domain.com'', ''ims001@new.domain.com'', ''{XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}''';
			PRINT N' 	- Simulate user ''ims001'' domain migration with UPN & AD GUID changes from ''current.domain.com'' to ''new.domain.com'' without performing actual operation';
			PRINT N'';
			PRINT N' * sp_migrateUserDomain ''SIMULATE'',''ims001'', ''ims002'',''current.domain.com'', NULL, NULL';
			PRINT N' 	- Simulate user ''ims001'' domain migration from ''current.domain.com'' to ''new.domain.com'' without performing actual operation';
			PRINT N'';
			PRINT N' * sp_migrateUserDomain ''UPDATE'',''ims001'', ''current.domain.com'', ''new.domain.com'', ''{XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}''';
			PRINT N' 	- Migrate user ''ims001'' from ''current.domain.com'' to ''new.domain.com'', update UPN to ''ims001@new.domain.com'' and AD GUID';
			PRINT N' 	  to ''{XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX}''';
			PRINT N'';
			PRINT N' * sp_migrateUserDomain ''UPDATE'',''ims001'', ''ims002'',''current.domain.com'', NULL, NULL';
			PRINT N' 	- Migrate user ''ims001'' from ''current.domain.com'' to ''new.domain.com'' without update UPN & AD GUID';
			
		END;
	END;
END;
